<?php
/*
Plugin Name: ETWP Affiliate Autolinks
Description: Rewrites specific URLs to affiliate links using a custom post type.
Version: 1.0
Author: EasyToolsWP.com
*/

if (!defined('ABSPATH')) {
    exit; // Prevent direct access
}

class ETWPAffiliateAutolinks {

    public function __construct() {
        add_action('init', array($this, 'create_custom_post_type'));
        add_action('add_meta_boxes', array($this, 'add_meta_boxes'));
        add_action('save_post', array($this, 'save_meta_box_data'));
        add_filter('the_content', array($this, 'replace_urls_in_content'));
        add_filter('manage_affiliate_autolink_posts_columns', array($this, 'set_custom_columns'));
        add_action('manage_affiliate_autolink_posts_custom_column', array($this, 'custom_column_data'), 10, 2);
    }

    // Create custom post type
    public function create_custom_post_type() {
        register_post_type('affiliate_autolink',
            array(
                'labels' => array(
                    'name' => __('Affiliate Partners'),
                    'singular_name' => __('Affiliate Partner')
                ),
                'public' => false,
                'show_ui' => true,
                'has_archive' => false,
                'supports' => array('title'),
                'menu_position' => 20,
                'menu_icon' => 'dashicons-admin-links',
            )
        );
    }

    // Add metaboxes to the custom post type
    public function add_meta_boxes() {
        add_meta_box(
            'affiliate_autolink_meta_box',
            __('Affiliate Link Details'),
            array($this, 'render_meta_box'),
            'affiliate_autolink',
            'normal',
            'high'
        );
    }

    // Display input fields in the metabox
    public function render_meta_box($post) {
        wp_nonce_field('save_affiliate_autolink_data', 'affiliate_autolink_meta_box_nonce');

        $url = get_post_meta($post->ID, '_original_url', true);
        $aff_link = get_post_meta($post->ID, '_affiliate_link', true);

        echo '<p><label for="original_url">' . __('Original Domain:') . '</label>';
        echo '<input type="text" id="original_url" name="original_url" value="' . esc_attr($url) . '" size="50" placeholder="https://www.example.com/" /></p>';

        echo '<p><label for="affiliate_link">' . __('Affiliate Link Template:') . '</label>';
        echo '<input type="text" id="affiliate_link" name="affiliate_link" value="' . esc_attr($aff_link) . '" size="50" placeholder="https://www.newaffiliate.com/campaign?product=(url)" /></p>';
    }

    // Save data from the metabox
    public function save_meta_box_data($post_id) {
        if (!isset($_POST['affiliate_autolink_meta_box_nonce'])) {
            return;
        }
        if (!wp_verify_nonce($_POST['affiliate_autolink_meta_box_nonce'], 'save_affiliate_autolink_data')) {
            return;
        }
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }
        if (!current_user_can('edit_post', $post_id)) {
            return;
        }

        if (isset($_POST['original_url'])) {
            update_post_meta($post_id, '_original_url', sanitize_text_field($_POST['original_url']));
        }
        if (isset($_POST['affiliate_link'])) {
            update_post_meta($post_id, '_affiliate_link', sanitize_text_field($_POST['affiliate_link']));
        }
    }

    // Rewrite URLs in the content
    public function replace_urls_in_content($content) {
        $args = array(
            'post_type' => 'affiliate_autolink',
            'posts_per_page' => -1,
            'post_status' => 'publish'
        );

        $autolinks = get_posts($args);

        if ($autolinks) {
            foreach ($autolinks as $autolink) {
                $original_url = get_post_meta($autolink->ID, '_original_url', true);
                $aff_link_template = get_post_meta($autolink->ID, '_affiliate_link', true);

                if (!empty($original_url) && !empty($aff_link_template)) {
                    // Find and replace all URLs starting with original_url
                    $pattern = '#\b' . preg_quote($original_url, '#') . '([^\s\'"<>]*)#';
                    $content = preg_replace_callback($pattern, function($matches) use ($aff_link_template) {
                        $full_url = $matches[0];
                        return str_replace('(url)', $full_url, $aff_link_template);
                    }, $content);
                }
            }
        }

        return $content;
    }

    // Add columns in the admin list view
    public function set_custom_columns($columns) {
        $columns['original_url'] = __('Original Domain');
        $columns['affiliate_link'] = __('Affiliate Link Template');
        return $columns;
    }

    // Display data in custom columns
    public function custom_column_data($column, $post_id) {
        switch ($column) {
            case 'original_url':
                echo esc_html(get_post_meta($post_id, '_original_url', true));
                break;
            case 'affiliate_link':
                echo esc_html(get_post_meta($post_id, '_affiliate_link', true));
                break;
        }
    }
}

new ETWPAffiliateAutolinks();
